#!/usr/bin/perl -w
use CGI qw/:standard :html3 :netscape/;
use strict;
use vars qw/%ClassInfo $before $after/;

%ClassInfo = ();

my $template = '';
{
 local $/ = undef;
 open( F, "schedule_tmpl.html" ) or die "Unable to open template file.\n";
 $template = <F>;
 close( F );
}

($before, $after) = split /<cgi_content\/>/, $template;

$_ = param('action');

CASE: 
 {
  /^add/i    and do { add_schedule();     last CASE; };
  /^drop/i   and do { drop_schedule();    last CASE; };
  /^update/i and do { update_schedule();  last CASE; };
    # default
  view_schedule();
 }


sub  get_class_info
 {
  my $data = <<'EOD';
3304 & Introduction to Team Programming  & 9-10 am     & MWF & 123 & Xavier
3320 & Introduction to Computer Graphics &  3-4 pm     & MWF & 115 & Zimmerman
3330 & Security - Timing Attacks         & 8:30-10 am  & TTh & 123 & Brown
4301 & Creative Problem Solving          & 10-11 am    & MWF & 123 & Sculley
4320 & Just In Time Compiler Theory      & 2:30-4 pm   & TTh & 203 & Brown
4355 & Genetic Algorithms                & 9-10 am     & MWF & 123 & Bashir
4360 & Programming in a Hetergeneous Environment & 9-10 am & MWF & 203 & Franklin
4365 & Triage as a Development Skill     & 10-11:30 am & TTh & 210 & Franklin
5302 & Design to Exceed Expectations     & 11:30-12 pm & TTh & 203 & Xavier
5350 & Ultimate User Interfaces          & 9-10 am     & MWF & 215 & Zimmerman
5404 & Detecting Intruders               & 1-3 pm      & TTh & 203 & Sculley
6330 & Human Genome Project              & 5:30-7 pm   & TTh & 123 & Crusher
EOD

  foreach(split /\n/, $data)
   {
    next if /^\s*$/;

    my @f = split /\s+&\s+/;
    $ClassInfo{$f[0]} = [ @f[1..5] ];
   }
 }


sub  print_schedule
 {
  my @classes = @_;
  my @rows    = ();
  
  if(@classes)
   {
    @rows = ( TR( th( [ qw(Class Name Time Days Room Professor)] ) ) );
    
    get_class_info();

    foreach my $class (@classes)
     {
      push @rows, TR( td( [ $class, @{$ClassInfo{$class}} ] ) );
     }

    print table( { -cellspacing=>0, -border => 1 }, @rows );
   }
  else
   {
    print h2( 'You have not yet signed up for classes.' ), br();
   }

  print start_form, submit( -name=>'action', -value=> 'Add Classes' ),
                    submit( -name=>'action', -value=> 'Drop Classes' ),
        end_form;
 }


sub  view_schedule
 {
  my @classes = cookie('classes');
  print header, $before;

  print_schedule( @classes );
  
  print $after;
 }


sub  add_schedule
 {
  my @classes = cookie('classes');
  my @classlist = ();

  print header, $before;

  get_class_info();

  print h2( 'Select classes to add.' );

  my %Filter = map { $_ => 1 } @classes;
  
  my @rows = ( TR( th( [ qw(Add Class Name Time Days Room Professor)] ) ) );
    
  foreach my $class (sort keys %ClassInfo)
   {
    next if exists $Filter{$class};
    
    push @rows, TR( td( checkbox( -name => $class, -value => 'add', -label => '' ) ),
                    td( [ $class, @{$ClassInfo{$class}} ] ) );
   }

  print start_form, table( { -cellspacing=>0, -border => 1 }, @rows ), br(),
                    submit( -name=>'action', -value=> 'Update My Schedule' ),
                    submit( -name=>'action', -value=> 'View My Schedule' ),
        end_form;
  
  print $after;
 }




sub  drop_schedule
 {
  my @classes = cookie('classes');

  print header, $before;

  get_class_info();

  print h2( 'Select classes to drop.' );

  my @rows = ( TR( th( [ qw(Add Class Name Time Days Room Professor)] ) ) );
    
  foreach my $class (@classes)
   {
    push @rows, TR( td( checkbox( -name => $class, -value => 'delete',
                                  -label => '' ) ),
                    td( [ $class, @{$ClassInfo{$class}} ] ) );
   }

  print start_form, table( { -cellspacing=>0, -border => 1 }, @rows ), br(),
                    submit( -name=>'action', -value=> 'Update My Schedule' ),
                    submit( -name=>'action', -value=> 'View My Schedule' ),
        end_form;
  
  print $after;
 }



sub  update_schedule
 {
  my @classes  = cookie('classes');
  my @added    = ();
  my @deleted  = ();
  my @warnings = ();

  get_class_info();

  my %Filter = map { $_ => 1 } @classes;
  my %Times  = map { $ClassInfo{$_}->[1].$ClassInfo{$_}->[2] => 1 } @classes;
  
  foreach my $class (sort keys %ClassInfo)
   {
    if('add' eq param( $class ))
     {
      if(exists $Filter{$class})
       {
        push @warnings, "$class already on your schedule.";
       }
      elsif(exists $Times{$ClassInfo{$class}->[1].$ClassInfo{$class}->[2]})
       {
        push @warnings, "$class conflicts with a class on your schedule.";
       }
      else
       {
        push @added, $class;
        $Times{$ClassInfo{$class}->[1].$ClassInfo{$class}->[2]} = 1;
       }
     }
    if('delete' eq param( $class ))
     {
      unless(exists $Filter{$class})
       {
        push @warnings, "$class not on your schedule.";
       }
      else
       {
        push @deleted, $class ;
       }
     }
   }

  %Filter = map { $_ => 1 } @deleted;
  @classes = sort grep { !exists $Filter{$_} } @classes, @added;

  my $classcookie = cookie( -name => 'classes', -value => \@classes );
  print header( -cookie => [ $classcookie ] ), $before;

  if(@warnings)
   {
    print h2( 'Warnings:' ),
          ul( li( [ map { font({ -color => 'red' }, $_ ) } @warnings ] ) ),
          br();
   }

  if(@added or @deleted)
   {
    print h2( 'Schedule changed successfully' );
   }
  else
   {
    print h2( 'Schedule not changed' );
   }

  print_schedule( @classes );
  
  print $after;
 }
